@@ -8,11 +8,11 @@ import json
 import logging
 import os
 import re
+import socket
 import sys
 from threading import Timer
 import time
 import urllib
-import subprocess
 import webbrowser
 
 from flask import (Flask, render_template, send_from_directory,
@@ -53,6 +53,7 @@ app.config.from_envvar('WEBDIFF_CONFIG', silent=True)
 A_DIR = None
 B_DIR = None
 DIFF = None
+PORT = None
 
 if app.config['TESTING'] or app.config['DEBUG']:
     handler = logging.StreamHandler()
@@ -66,6 +67,7 @@ if app.config['TESTING'] or app.config['DEBUG']:
         log.setLevel(logging.DEBUG)
         log.addHandler(handler)
 else:
+    # quiet down werkzeug -- no need to log every request.
     logging.getLogger('werkzeug').setLevel(logging.ERROR)
 
 
@@ -146,7 +148,10 @@ def get_contents(side):
 # Show the first diff by default
 @app.route("/")
 def index():
-    return render_template('heartbeat.html', src='/0')
+    if not 'NO_FRAME' in app.config:
+        return render_template('heartbeat.html', src='/0')
+    else:
+        return file_diff('0')
 
 
 @app.route("/<idx>")
@@ -169,7 +174,7 @@ def favicon():
 
 @app.route('/kill', methods=['POST'])
 def kill():
-    logging.info("Trying to die")
+    if 'STAY_RUNNING' in app.config: return
     func = request.environ.get('werkzeug.server.shutdown')
     if func is None:
         raise RuntimeError('Not running with the Werkzeug Server')
@@ -178,27 +183,47 @@ def kill():
 
 
 def open_browser():
-    webbrowser.open_new_tab('http://localhost:5000')
+    global PORT
+    if not 'NO_OPEN_BROWSER' in app.config:
+        webbrowser.open_new_tab('http://localhost:%s' % PORT)
+
+
+def usage_and_die():
+    sys.stderr.write(
+'''Usage: webdiff <left_dir> <right_dir>
+
+When using git difftool, make sure you run "git difftool -d".
+''')
+    sys.exit(1)
+
+
+def pick_a_port():
+    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    sock.bind(('localhost', 0))
+    port = sock.getsockname()[1]
+    sock.close()
+    return port
 
 
 def run():
-    global A_DIR, B_DIR, DIFF
+    global A_DIR, B_DIR, DIFF, PORT
     assert len(sys.argv) == 3
     A_DIR = sys.argv[1]
     B_DIR = sys.argv[2]
     if not os.path.isdir(A_DIR) or not os.path.isdir(B_DIR):
-        sys.stderr.write('Make sure you run git difftool -d\n')
-        sys.exit(1)
+        usage_and_die()
+
+    PORT = pick_a_port()
 
     if app.config['TESTING'] or app.config['DEBUG']:
         sys.stderr.write('Diffing:\nA: %s\nB: %s\n\n' % (A_DIR, B_DIR))
 
-    sys.stderr.write('''Serving diffs on http://localhost:5000
+    sys.stderr.write('''Serving diffs on http://localhost:%s
 Close the browser tab or hit Ctrl-C when you're done.
-''')
+''' % PORT)
     DIFF = find_diff(A_DIR, B_DIR)
     Timer(0.1, open_browser).start()
-    app.run(host='0.0.0.0')
+    app.run(host='0.0.0.0', port=PORT)
 
 
 if __name__ == '__main__':
